
% Demo script for the dictionary learning (DL) problem in the tensor form
%
% This script illustrates the use of the DL algorithm
% implemented in the functions DicLear_T, Lea_Algorithm_Tensor_D2 and
% Lea_Algorithm__Tensor_Dinf .
% The user can easily modify the script for
% other tests problems, patch sizes etc..
%

% Inputs:
% TImage: The test image
% patch_size: The size of training patchs
% t: Number of training patches
% s: Number of dictionary elements
% DSet: The compact and convex set which the dictionary belongs to
% Lambda: The regularization parameter
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,

% Outputs:
% D: The tensor dictionary of size: patch_size(1) \times s \times
% patch_size(2)
% H: The representation tensor of size: s \times t \times patch_size(2)

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com

clear all;
close all;
clc;

% Set up the necessary paths
addpath /home/ssol/DLCT-ToolBox/TestImages
%% Set up the problem

% Choose a test image
% The test image options are: Peppers,Matches,Binary,D53,Zirconium,Steel
% Default values = 'Peppers';

TImage = 'Peppers';

% Choose the training patch size p,r 
% The training patches can be square or rectangular
% The author suggests 5 by 5 (p=r=5), 10 by 10 (p=r=10) and 
% 20 by 20 (p=r=20) patches;
% Default values = 10 by 10;

patch_size=[5 5];

% You can choose number of training patches to be considered in dictionary 
% learning process.
% The default values are set to be 10000; 
% The maximum is the number of all the overlapping 
% patches in the test images and it varies depending on the
% test image and patch sizes-Note that this number could be huge-
% The extracted over-lapping training patches are randomly shuffeled.
t = 20000; 

% Choose number of dictionary elements
% The author suggests an over-representaion factor of 2,3, i.e., 
% s=2*p*r, s=3*p*r.
% Default values = 200;
s = 50; 

% Choose the regularization parameter Lambda
% A range of different values for Lambda can also be chosen
% The default value is Lambda=logspace(0.5,0.5,1)=3.1623

Lambda=logspace(0.5,0.5,1);

% Choose the compact and convex set which the dictionary belongs to
% The options are: D_inf and D_2, 
% D_inf corresponds to box constraints, and D_2 is a spherical 
% non-negative sector of the 2-norm ball with radius sqrt(p*r)
% The default value is D_2
DSet = 'D_2';

% The display option on the screen
% When set to 0 no display on the screen
% When set to 1 printes progress of the algorithm
% default = 0,
Outdisplay = 1;
%% The solution and its illustration


[D,H] = DicLear_T(TImage,patch_size,s,Lambda,DSet,t,Outdisplay);
 
% Illustrate a logarithmic histogram of H
 
figure, hist(H(:),100);
ph = get(gca,'children');
N_child = length(ph);
for j = 1:N_child
    vn = get(ph(j),'Vertices');
    vn(:,2) = vn(:,2) + 1;
    set(ph(j),'Vertices',vn)
 end

set(gca,'yscale','log')

title('The logarithmic histogram of vec(H)', 'FontSize', 16)
xlabel('Bin values','FontSize', 16)
ylabel('Frequency','FontSize', 16)

D_Squeezed = zeros(patch_size(1),patch_size(2),s);
for j=1:s
    D_Squeezed(:,:,j)=squeeze(D(:,j,:));
end
figure, montageplot(permute(D_Squeezed,[3 2 1]),[],ceil(sqrt(s)),...
    floor(sqrt(s)));


% Save the dictionary in the correct path
%save('/home/ssol/DLCT-ToolBox/TestResults/DicTensor.mat','D');