function [ W,H,Rs1,Rs2,Rs3,Rs4,Rs5,fobjec,ResF] = ...
  Lea_Algorithm_Dinf(X,tol,maxiter,noc,Cof_Lambda,U,V,Cof_Alpha,Outdisplay)  

   % function Lea_Algorithm_D2 performs the ADMM algortihm in k iterations
   % for the dictionary learning problem in the Matrix form, where the
   % dictionary D belongs to the set D_inf
   
% Usage:
% [ W,H,Rs1,Rs2,Rs3,Rs4,Rs5,fobjec,ResF] = ...
%    Lea_Algorithm_Dinf(X,tol,maxiter,noc,Cof_Lambda,U,V,Cof_Alpha) ; 

% Input:
% X: Training matrix,
% tol: The tolerance for the ADMM convergance
% maxiter: total number of ADMM iterations
% noc: The number of dictionary elements,
% Cof_Lambda: the regularization parameter Lambda in the dictionary
% learning problem formulation
% U,V: The initial value for the auxiliary variables in the ADMM algorithm.
% Cof_Alpha: The agumented lagrangian parameter
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,

% Output:
% W: The dictionary matrix for the corresponding lambda
% H: The representation matrix for the corresponding lambda
% fobjec: The objective function value for each iteration of the ADMM
% algorithm: 1/2||Y_DH||^2_F+\lambda||H||_{sum}
% ResF: The residual value for each iteration of the ADMM
% algorithm: ||Y_DH||_F
% Rs1,Rs2,Rs3,Rs4,Rs5: The stopping critera values at each iteration 
% of the ADMM algorithm

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com
 

[m,n] = size(X);


Lag_PI = zeros(noc,n);
Lag_Delta = zeros(m,noc);
   
H = V;


%%
fobjec=zeros(maxiter,1);
Rs1=zeros(maxiter,1);
Rs2=zeros(maxiter,1);
Rs3=zeros(maxiter,1);
Rs4=zeros(maxiter,1);
Rs5=zeros(maxiter,1);
ResF=zeros(maxiter,1);
%%

for k=1:maxiter
    
   
    % W update 
    W=U-1/Cof_Alpha*Lag_Delta;
    
   %  projection when we are considering 0<=w_ij<=1
    W=max(W,0);
    W=min(W,1);
   
    
    % V update
    V=(U'*U+Cof_Alpha*eye(noc,noc))\(U'*X+Lag_PI+Cof_Alpha*H);
    
    % H update
    
    H= soft_thresh(Cof_Alpha,Cof_Lambda,V,Lag_PI,noc,n);
  
    % U update
    U = (X*V'+Cof_Alpha*W + Lag_Delta)/(V*V'+Cof_Alpha*eye(noc,noc));

    % Lagrangian variable update
    Lag_Delta = Lag_Delta + Cof_Alpha*(W-U);
    Lag_PI = Lag_PI + Cof_Alpha*(H-V);


    %% 
    Rs1(k)=norm(W(:)-U(:),'inf')/max(1,norm(W(:),'inf'));
    Rs2(k) = norm(H(:)-V(:),'inf')/max(1,norm(H(:),'inf'));
    Rs=zeros(noc,1);
    
    for i=1:noc
        Rs(i,1)=norm(W(:,i),'inf');
    end

    % Dual Resudials
    Comp = X-W*H;
    ResF(k)=norm(Comp,'fro');
    tempp0=W'*Comp;
    tempp1=Comp*H';
    Rs4(k) = norm(tempp0+Lag_PI,'inf')/max(1,norm(tempp0,'inf'));
    Rs5(k) = norm(tempp1+Lag_Delta,'inf')/max(1,norm(tempp1,'inf'));

    Rs3(k)=max(1-Rs);
     
    fobjec(k)=1/2*(norm(Comp,'fro')^2)+Cof_Lambda*norm(H(:),1);
    if (Rs1(k)<= tol && Rs2(k)<=tol && Rs3(k)<=tol && Rs4(k)<=tol && ...
            Rs5(k)<=tol && fobjec(k) <= fobjec(1) )
        if Outdisplay==1
           disp('Algorithm is converged. The tolerance reached') 
        end
    break
    end
    if Outdisplay==1
    disp(['The ADMM iteration number =', num2str(k)])
    disp(['The residual at iteration ', num2str(k),...
        ' i.e., ||Y-DH||_F = ',num2str(ResF(k)) ])
    end

end

end
%%
function V = soft_thresh(Cof_Beta,Cof_Lambda,V,Lag_PI,noc,n)

th = Cof_Lambda/(Cof_Beta*2);  
x = zeros(noc*n,1);

B = (V - Lag_PI./Cof_Beta); 
B = reshape(B,noc*n,1);

k = find(B > th);
x(k) = B(k)-th;

k = find(abs(B) <= th);
x(k) = 0;

k = find (B < -th);
x(k) = 0;
V = reshape(x,noc,n);

end