% MAET_demo

% This demo script file illustrates how to compute the approximation
% error of a test problem in the cone defined by the dictionary implemented
% in the functions MAE_T and Linear_TenLSQ_BestRec
% The user can easily modify the script for
% other tests problems, patch sizes etc...
% This code handle various dictionaries for a range of values for lambda.
% This codes uses the TFOCS optimization solver (http://cvxr.com/tfocs/)
% make sure TFOCS is properly installed on your machine
%
% Inputs:
% TrImage: The test image, options are: 'Peppers', 'Matches', 
% 'Binary', 'Zirconium', 'Steel', 'D53'
% patch_size: The patch sizes of the dictionary elements, does not 
% necessarily need to be provided 
% default = [size(D,1),size(D,3)]
% D: The given(learned) dictionary, note that the dictionary should be
% obtained from a training image similar to the test image
% Lambda: The regularization parameter in the dictionary learning problem
% formulation, note that it should be consistence with the dictionary D
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,

% Outputs:
% Appr_Error_Mean: Mean approximation error
% Appr_Error: The vector of approximation error for each block in the image
% Alph: The best representation/approximation of the jth block in the cone

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com

clear all;
close all;
clc;


% Set the necessary paths
addpath /home/ssol/TFOCS-1.3.1

addpath /home/ssol/DLCT-ToolBox/TestImages
addpath /home/ssol/DLCT-ToolBox/TestResults

%% Set the test problem
% The test image, The test image, options are: 'Peppers', 'Matches', 
% 'Binary', 'Zirconium', 'Steel', 'D53'
TrImage = 'Peppers';

% Load the desired dictionary, note that the dictionary should be obtained
% from a similar training image
load('DicTensor.mat','D');

% Choose the regularization parameter Lambda in the dictionary learning
% problem
% A range of different values for Lambda can also be chosen
Lambda = logspace(0.5,0.5,1);

% The patch sizes of the dictionary elements,
patch_size = [5,5]; 

% The display option on the screen
% When set to 0 no display on the screen
% When set to 1 printes progress of the algorithm
% default = 0,
Outdisplay = 1;

[ Appr_Error_Mean,Appr_Error,Alph ,Norm1Alph ,SparsityAlph] = ...
      MAE_T( TrImage, D, Lambda,patch_size,Outdisplay);
  
  figure, hist(Appr_Error,100)   