function [ pv ] = dykstra( v )
% Dykstra Projection Algorithm
% Dykstra's algorithm is a method that computes a point in the intersection
% of convex sets, and is a variant of the alternating projection method 
% (also called the projections onto convex sets method). In its simplest 
% form, the method finds a point in the intersection of two convex sets by 
% iteratively projecting onto each of the convex set.
% This function computes the projection of a vector v into the intersection
% of the non-negative orthant and the 2-norm ball with the radius 
% sqrt(length(v))

% Input:
% v: a given vector
% Output:
% pv: projection of the vector v into intersection of the non-negative 
% orthant and the 2-norm ball with the radius sqrt(length(v))
% Sara Soltani, ssol@dtu.dk, 28.July.2015

tol=1e-3;
        p=zeros(size(v));
        q=p; 
       for i=1:100  
         y=max(0,v+p);
         p=v+p-y;
         v=y+q;
         v = v / max(norm(v,2)/sqrt(size(v,1)*size(v,2)), 1);
         q=y+q-v;
         if norm(v-y)<= tol  
             break
             % The algorithm terminates when the sequences converges 
             % to the solution
         end
       end

      pv=v;

 
end

